@@ -10,6 +10,7 @@ import random
 import copy
 import queue
 import collections
+import socket
 from hashlib import sha256
 from threading import Thread, get_ident
 from nem_ed25519.base import Encryption
@@ -20,7 +21,8 @@ from .tool.utils import StackDict, EventIgnition, JsonDataBase, QueueSystem
 from .tool.upnpc import UpnpClient
 
 LOCAL_IP = UpnpClient.get_localhost_ip()
-GLOBAL_IP = UpnpClient.get_global_ip()
+GLOBAL_IPV4 = UpnpClient.get_global_ip()
+GLOBAL_IPV6 = UpnpClient.get_global_ip_ipv6()
 STICKY_LIMIT = 2
 
 # Constant type
@@ -49,7 +51,7 @@ class PeerClient:
 
     def __init__(self, listen=15, f_local=False):
         assert V.DATA_PATH is not None, 'Setup p2p params before PeerClientClass init.'
-        self.p2p = Core(host='127.0.0.1' if f_local else '', listen=listen)
+        self.p2p = Core(host='localhost' if f_local else None, listen=listen)
         self.broadcast_que = QueueSystem()  # BroadcastDataが流れてくる
         self.event = EventIgnition()  # DirectCmdを受け付ける窓口
         self.__broadcast_uuid = collections.deque(maxlen=listen*20)  # Broadcastされたuuid
@@ -65,7 +67,7 @@ class PeerClient:
         self.p2p.close()
         self.f_stop = True
 
-    def start(self, f_stabilize=True):
+    def start(self, s_family=socket.AF_UNSPEC, f_stabilize=True):
         processing_que = self.p2p.core_que.create()
         broadcast_que = queue.LifoQueue()
 
@@ -115,7 +117,7 @@ class PeerClient:
             logging.info("Close broadcast.")
 
         self.f_running = True
-        self.p2p.start()
+        self.p2p.start(s_family)
         if f_stabilize:
             Thread(target=self.stabilize, name='Stabilize', daemon=True).start()
         # Processing
@@ -541,9 +543,11 @@ class PeerClient:
         time.sleep(5)
         logging.info("start stabilize.")
         ignore_peers = {
-            (GLOBAL_IP, V.P2P_PORT),
+            (GLOBAL_IPV4, V.P2P_PORT),
+            (GLOBAL_IPV6, V.P2P_PORT),
             (LOCAL_IP, V.P2P_PORT),
-            ('127.0.0.1', V.P2P_PORT)}
+            ('127.0.0.1', V.P2P_PORT),
+            ('::1', V.P2P_PORT)}
         if len(self.peers) == 0:
             logging.info("peer list is zero, need bootnode.")
         else:
@@ -573,12 +577,10 @@ class PeerClient:
         need_connection = 3
         while not self.f_stop:
             count += 1
-            if not (self.p2p.listen * 1 // 3 < len(self.p2p.user) < self.p2p.listen * 2 // 3):
-                time.sleep(5)
-            elif count < 5 and len(self.p2p.user) < need_connection:
-                time.sleep(2)
-            elif len(self.p2p.user) < need_connection:
+            if len(self.p2p.user) < need_connection:
                 sticky_nodes.clear()
+                time.sleep(2)
+            elif len(self.p2p.user) < self.p2p.listen // 3:
                 time.sleep(5)
             elif count % 24 == 1:
                 time.sleep(5 * (1 + random.random()))
@@ -692,6 +694,7 @@ class PeerClient:
                 logging.debug("ConnectionError {}".format(e))
             except Exception as e:
                 logging.debug("Stabilize {}".format(e), exc_info=True)
+        logging.error("Get out from loop of stabilize.")
 
     @staticmethod
     def broadcast_check(data):
